"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFrameBounds = exports.getFrameVirtualNode = exports.FramesRenderer = void 0;
var native_renderer_1 = require("./native-renderer");
var paperclip_utils_1 = require("paperclip-utils");
var utils_1 = require("./utils");
var dom_patcher_1 = require("./dom-patcher");
var cssom_patcher_1 = require("./cssom-patcher");
var paperclip_common_1 = require("paperclip-common");
var FramesProxy = /** @class */ (function () {
    function FramesProxy(_renderer, bindState) {
        this._renderer = _renderer;
        this.namespaceURI = null;
        this._childNodes = [];
        this._importedStyles = [];
        this._importedNativeStyles = [];
        this._store = new paperclip_common_1.ImmutableStore({ frames: [] }, bindState);
    }
    FramesProxy.prototype.getState = function () {
        return this._store.getState();
    };
    Object.defineProperty(FramesProxy.prototype, "childNodes", {
        get: function () {
            return this._childNodes;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FramesProxy.prototype, "importedStyles", {
        get: function () {
            return this._importedStyles;
        },
        enumerable: false,
        configurable: true
    });
    FramesProxy.prototype.setMainStyle = function (style) {
        this._mainStyle = style;
        var nativeStyle = (0, native_renderer_1.createNativeStyleFromSheet)(style, this._renderer.getDOMFactory(), this._renderer.getUrlResolver());
        for (var _i = 0, _a = this.getState().frames; _i < _a.length; _i++) {
            var frame = _a[_i];
            removeAllChildren(frame._mainStylesContainer);
            frame._mainStylesContainer.appendChild(nativeStyle.cloneNode(true));
        }
    };
    FramesProxy.prototype.applyStylePatches = function (mutations, uri) {
        var styleIndex = this._importedStyles.findIndex(function (style) {
            return style.uri === uri;
        });
        // first do the frames
        for (var _i = 0, _a = this.getState().frames; _i < _a.length; _i++) {
            var frame = _a[_i];
            var styleElement = (styleIndex !== -1
                ? frame._importedStylesContainer.childNodes[styleIndex]
                : frame._mainStylesContainer.childNodes[0]);
            (0, cssom_patcher_1.patchCSSOM)(styleElement.sheet, mutations, this._renderer.getUrlResolver());
        }
        // note we patch the virt objects here since native styles don't parse text unless
        // mounted so we don't have access to that
        if (styleIndex === -1) {
            this._mainStyle = (0, paperclip_utils_1.patchCSSSheet)(this._mainStyle, mutations);
        }
        else {
            this._importedStyles[styleIndex] = __assign(__assign({}, this._importedStyles[styleIndex]), { sheet: (0, paperclip_utils_1.patchCSSSheet)(this._importedStyles[styleIndex].sheet, mutations) });
        }
    };
    FramesProxy.prototype.updateImportedStyles = function (newStyles, removeStyleUris) {
        if (removeStyleUris === void 0) { removeStyleUris = []; }
        var rmIndices = [];
        // need to remove sheet in case we're looking to replace
        for (var i = this._importedStyles.length; i--;) {
            var style = this._importedStyles[i];
            if (removeStyleUris.includes(style.uri)) {
                rmIndices.push(i);
                this._importedStyles.splice(i, 1);
                this._importedNativeStyles.splice(i, 1);
            }
        }
        for (var _i = 0, rmIndices_1 = rmIndices; _i < rmIndices_1.length; _i++) {
            var i = rmIndices_1[_i];
            for (var _a = 0, _b = this.getState().frames; _a < _b.length; _a++) {
                var frame = _b[_a];
                var nativeSheet = frame._importedStylesContainer.childNodes[i];
                nativeSheet.remove();
            }
        }
        // finally append
        for (var _c = 0, newStyles_1 = newStyles; _c < newStyles_1.length; _c++) {
            var info = newStyles_1[_c];
            var sheet = info.sheet, index = info.index;
            this._importedStyles.splice(index, 0, info);
            var nativeSheet = (0, native_renderer_1.createNativeStyleFromSheet)(sheet, this._renderer.getDOMFactory(), this._renderer.getUrlResolver());
            this._importedNativeStyles.splice(index, 0, nativeSheet);
            for (var _d = 0, _e = this.getState().frames; _d < _e.length; _d++) {
                var frame = _e[_d];
                var beforeChild = frame._importedStylesContainer.childNodes[index];
                var sheet_1 = nativeSheet.cloneNode(true);
                if (beforeChild) {
                    frame._importedStylesContainer.insertBefore(sheet_1, beforeChild);
                }
                else {
                    frame._importedStylesContainer.appendChild(sheet_1);
                }
            }
        }
    };
    FramesProxy.prototype.appendChild = function (child) {
        this.insert(child, this.getState().frames.length);
    };
    FramesProxy.prototype.insertBefore = function (child, existing) {
        this.insert(child, this._childNodes.findIndex(function (_child) { return _child === existing; }));
    };
    FramesProxy.prototype.removeChild = function (child) {
        var index = this._childNodes.findIndex(function (_child) { return _child === child; });
        this._childNodes.splice(index, 1);
        this._store.update(function (state) {
            state.frames = (0, utils_1.arraySplice)(state.frames, index, 1);
        });
    };
    FramesProxy.prototype.insert = function (child, index) {
        var _importedStylesContainer = this._renderer
            .getDOMFactory()
            .createElement("div");
        for (var _i = 0, _a = this._importedNativeStyles; _i < _a.length; _i++) {
            var style = _a[_i];
            _importedStylesContainer.appendChild(style.cloneNode(true));
        }
        var _mainStylesContainer = this._renderer
            .getDOMFactory()
            .createElement("div");
        if (this._mainStyle) {
            _mainStylesContainer.appendChild((0, native_renderer_1.createNativeStyleFromSheet)(this._mainStyle, this._renderer.getDOMFactory(), this._renderer.getUrlResolver()));
        }
        var _mount = this._renderer.getDOMFactory().createElement("div");
        var stage = this._renderer.getDOMFactory().createElement("div");
        stage.appendChild(_importedStylesContainer);
        stage.appendChild(_mainStylesContainer);
        stage.appendChild(_mount);
        _mount.appendChild(child);
        this._childNodes.splice(index, 0, child);
        this._store.update(function (state) {
            state.frames = (0, utils_1.arraySplice)(state.frames, index, 0, {
                stage: stage,
                _importedStylesContainer: _importedStylesContainer,
                _mainStylesContainer: _mainStylesContainer,
                _mount: _mount,
                sourceUri: null
            });
        });
    };
    return FramesProxy;
}());
/**
 * splits a file out into frames that can be
 */
var FramesRenderer = /** @class */ (function () {
    function FramesRenderer(targetUri, _resolveUrl, domFactory, connectState) {
        var _this = this;
        this._resolveUrl = _resolveUrl;
        this._dependencies = [];
        this._onFramesProxyChange = function (frameState) {
            _this._store.update(function (newState) {
                newState.frames = frameState.frames;
            });
        };
        /**
         */
        this.handleEngineDelegateEvent = function (event) {
            switch (event.kind) {
                case paperclip_utils_1.EngineDelegateEventKind.ChangedSheets: {
                    if (event.uri === _this.targetUri) {
                        _this._dependencies = event.data.allImportedSheetUris;
                        _this._framesProxy.updateImportedStyles(event.data.newSheets, event.data.removedSheetUris);
                    }
                    break;
                }
                case paperclip_utils_1.EngineDelegateEventKind.Loaded: {
                    if (event.data.kind === paperclip_utils_1.EvaluatedDataKind.PC) {
                        if (event.uri === _this.targetUri) {
                            _this._dependencies = event.data.allImportedSheetUris;
                            _this.initialize(event.data);
                        }
                    }
                    break;
                }
                case paperclip_utils_1.EngineDelegateEventKind.Evaluated: {
                    if (event.data.kind === paperclip_utils_1.EvaluatedDataKind.PC) {
                        // console.log("EV", this._dependencies, this._dependencies.includes(event.uri), event.uri);
                        //   if ([
                        //     "file:///Users/crcn/Developer/work/capital/frontend/packages/design-system/src/fonts/Eina_03/font-family.pc",
                        //     "file:///Users/crcn/Developer/work/capital/frontend/packages/design-system/src/fonts/IBM_Plex_Sans/font-family.pc",
                        //     "file:///Users/crcn/Developer/work/capital/frontend/packages/design-system/src/fonts/Inter/font-family.pc",
                        //     "file:///Users/crcn/Developer/work/capital/frontend/packages/design-system/src/fonts/Supera_Gothic/font-family.pc",
                        //     "file:///Users/crcn/Developer/work/capital/frontend/packages/design-system/src/atoms.pc",
                        //     "file:///Users/crcn/Developer/work/capital/frontend/packages/design-system/src/utils.pc",
                        //     "file:///Users/crcn/Developer/work/capital/frontend/packages/design-system/src/Button.pc"
                        // ].includes(event.uri)) {
                        //   console.log("EVALUATED");
                        // }
                        //   if (this.targetUri.includes("frontend/packages/design-system/src/Modal.pc")) {
                        //     console.log("MODAL");;
                        //   }
                        // if (this._dependencies.toString().includes("node_modules") || event.uri.includes("node_modules")) {
                        //   console.log("WAI...");
                        // }
                        if (event.uri === _this.targetUri) {
                            _this._dependencies = event.data.allImportedSheetUris;
                        }
                        else if (_this._dependencies.includes(event.uri)) {
                            _this._framesProxy.updateImportedStyles([
                                {
                                    uri: event.uri,
                                    sheet: event.data.sheet,
                                    index: _this._dependencies.indexOf(event.uri)
                                }
                            ], [event.uri]);
                        }
                    }
                    break;
                }
                case paperclip_utils_1.EngineDelegateEventKind.Diffed: {
                    if (event.data.kind === paperclip_utils_1.DiffedDataKind.PC) {
                        if (_this.targetUri !== event.uri &&
                            !_this._dependencies.includes(event.uri)) {
                            break;
                        }
                        // Style patches need to happen _before_ patching frames since style elements don't have
                        // their sheets until their mounted
                        _this._framesProxy.applyStylePatches(event.data.sheetMutations, event.uri);
                        if (event.uri === _this.targetUri) {
                            _this._dependencies = event.data.allImportedSheetUris;
                            (0, dom_patcher_1.patchNativeNode)(_this._framesProxy, event.data.mutations, _this._domFactory, _this._resolveUrl);
                            _this.setPreview((0, paperclip_utils_1.patchVirtNode)(_this.getState().preview, event.data.mutations));
                        }
                    }
                    else if (event.data.kind === paperclip_utils_1.DiffedDataKind.CSS) {
                        if (!_this._dependencies.includes(event.uri)) {
                            break;
                        }
                        _this._framesProxy.applyStylePatches(event.data.mutations, event.uri);
                    }
                    break;
                }
            }
        };
        this._domFactory = domFactory || document;
        this._store = new paperclip_common_1.ImmutableStore({
            uri: targetUri,
            frames: []
        }, connectState);
        this._framesProxy = new FramesProxy(this, this._onFramesProxyChange);
    }
    FramesRenderer.prototype.getUrlResolver = function () {
        return this._resolveUrl;
    };
    FramesRenderer.prototype.getDOMFactory = function () {
        return this._domFactory;
    };
    FramesRenderer.prototype.setUrlResolver = function (value) {
        this._resolveUrl = value;
    };
    FramesRenderer.prototype.setPreview = function (preview) {
        this._store.update(function (newState) {
            newState.preview = preview;
        });
    };
    Object.defineProperty(FramesRenderer.prototype, "targetUri", {
        get: function () {
            return this._store.getState().uri;
        },
        enumerable: false,
        configurable: true
    });
    FramesRenderer.prototype.getState = function () {
        return this._store.getState();
    };
    FramesRenderer.prototype.initialize = function (_a) {
        var sheet = _a.sheet, importedSheets = _a.importedSheets, preview = _a.preview, allImportedSheetUris = _a.allImportedSheetUris;
        var children = preview.kind === paperclip_utils_1.VirtualNodeKind.Fragment ? preview.children : [preview];
        this.setPreview(preview);
        this._imp = importedSheets;
        this._framesProxy = new FramesProxy(this, this._onFramesProxyChange);
        this._dependencies = allImportedSheetUris;
        this._framesProxy.setMainStyle(sheet);
        for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
            var child = children_1[_i];
            var childNode = (0, native_renderer_1.createNativeNode)(child, this._domFactory, this._resolveUrl, null);
            this._framesProxy.appendChild(childNode);
        }
        this._framesProxy.updateImportedStyles(importedSheets);
    };
    FramesRenderer.prototype.getRects = function () {
        var rects = {};
        var _loop_1 = function (i, length_1) {
            var frame = this_1.getState().frames[i];
            var frameNode = (0, exports.getFrameVirtualNode)(frame, this_1.getState().frames, this_1.getState().preview);
            if (!frameNode) {
                return "continue";
            }
            var bounds = (0, exports.getFrameBounds)(frameNode);
            // mount child node _is_ the frame -- can only ever be one child
            (0, utils_1.traverseNativeNode)(frame._mount.childNodes[0], function (node, path) {
                if (node.nodeType === 1) {
                    var pathStr = path.length ? i + "." + path.join(".") : i;
                    if (pathStr) {
                        var clientRect = node.getBoundingClientRect();
                        rects[pathStr] = {
                            width: clientRect.width,
                            height: clientRect.height,
                            x: clientRect.left + bounds.x,
                            y: clientRect.top + bounds.y
                        };
                    }
                }
            });
            // include frame sizes too
            rects[i] = bounds;
        };
        var this_1 = this;
        for (var i = 0, length_1 = this.getState().frames.length; i < length_1; i++) {
            _loop_1(i, length_1);
        }
        return rects;
    };
    return FramesRenderer;
}());
exports.FramesRenderer = FramesRenderer;
var getFrameVirtualNode = function (frame, frames, preview) {
    if (!preview) {
        return null;
    }
    var children = preview.kind === paperclip_utils_1.VirtualNodeKind.Fragment ? preview.children : [preview];
    return children[frames.indexOf(frame)];
};
exports.getFrameVirtualNode = getFrameVirtualNode;
exports.getFrameBounds = (0, paperclip_utils_1.memoize)(function (node) {
    var annotations = (node.annotations && (0, paperclip_utils_1.computeVirtJSObject)(node.annotations)) || {};
    return __assign({ width: 1024, height: 768, x: 0, y: 0 }, (annotations.frame || {}));
});
var removeAllChildren = function (node) {
    while (node.childNodes.length) {
        node.removeChild(node.childNodes[0]);
    }
};
