"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchNativeNode = void 0;
var html_entities_1 = require("html-entities");
var native_renderer_1 = require("./native-renderer");
var paperclip_utils_1 = require("paperclip-utils");
var utils_1 = require("./utils");
var entities = new html_entities_1.Html5Entities();
var patchNativeNode = function (mount, mutations, factory, resolveUrl) {
    for (var _i = 0, mutations_1 = mutations; _i < mutations_1.length; _i++) {
        var mutation = mutations_1[_i];
        var target = getTarget(mount, mutation);
        var action = mutation.action;
        switch (action.kind) {
            case paperclip_utils_1.ActionKind.DeleteChild: {
                var child = target.childNodes[action.index];
                target.removeChild(child);
                break;
            }
            case paperclip_utils_1.ActionKind.InsertChild: {
                var newChild = (0, native_renderer_1.createNativeNode)(action.child, factory, resolveUrl, target.namespaceURI);
                if (action.index >= target.childNodes.length) {
                    target.appendChild(newChild);
                }
                else {
                    target.insertBefore(newChild, target.childNodes[action.index]);
                }
                break;
            }
            case paperclip_utils_1.ActionKind.ReplaceNode: {
                var parent_1 = target.parentNode;
                parent_1.insertBefore((0, native_renderer_1.createNativeNode)(action.replacement, factory, resolveUrl, parent_1.namespaceURI), target);
                target.remove();
                break;
            }
            case paperclip_utils_1.ActionKind.RemoveAttribute: {
                var element = target;
                element.removeAttribute(utils_1.ATTR_ALIASES[action.name] || action.name);
                break;
            }
            case paperclip_utils_1.ActionKind.SetAttribute: {
                var element = target;
                var aliasName = utils_1.ATTR_ALIASES[action.name] || action.name;
                var value = action.value || "";
                if (value.indexOf("file:") === 0) {
                    value = resolveUrl(value);
                }
                element.setAttribute(aliasName, value);
                break;
            }
            case paperclip_utils_1.ActionKind.SetText: {
                var text = target;
                text.nodeValue = entities.decode(action.value);
                break;
            }
        }
    }
};
exports.patchNativeNode = patchNativeNode;
var getTarget = function (mount, mutation) {
    return mutation.nodePath.reduce(function (current, i) { return current.childNodes[i]; }, mount);
};
