"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createNativeStyleFromSheet = exports.createNativeNode = exports.getNativeNodePath = void 0;
var html_entities_1 = require("html-entities");
var paperclip_utils_1 = require("paperclip-utils");
var utils_1 = require("./utils");
var entities = new html_entities_1.Html5Entities();
var XMLNS_NAMESPACE = "http://www.w3.org/2000/svg";
var getNativeNodePath = function (root, node) {
    var path = [];
    var current = node;
    while (current.parentNode !== root) {
        path.unshift(Array.prototype.indexOf.call(current.parentNode.childNodes, current));
        current = current.parentNode;
    }
    return path;
};
exports.getNativeNodePath = getNativeNodePath;
var createNativeNode = function (node, factory, resolveUrl, namespaceURI) {
    if (!node) {
        return factory.createTextNode("");
    }
    try {
        switch (node.kind) {
            case "Text": {
                var text = createNativeTextNode(node, factory);
                return text;
            }
            case "Element":
                return createNativeElement(node, factory, resolveUrl, namespaceURI);
            case "StyleElement":
                return (0, exports.createNativeStyleFromSheet)(node.sheet, factory, resolveUrl);
            case "Fragment":
                return createNativeFragment(node, factory, resolveUrl);
        }
    }
    catch (e) {
        return factory.createTextNode(String(e.stack));
    }
};
exports.createNativeNode = createNativeNode;
var _dummyStyle;
var ruleIsValid = function (ruleText) {
    if (typeof window === "undefined") {
        return true;
    }
    if (!_dummyStyle) {
        _dummyStyle = document.createElement("style");
        document.head.appendChild(_dummyStyle);
    }
    try {
        _dummyStyle.sheet.insertRule(ruleText, 0);
        _dummyStyle.sheet.deleteRule(0);
    }
    catch (e) {
        return false;
    }
    return true;
};
var createNativeStyleFromSheet = function (sheet, factory, resolveUrl) {
    var nativeElement = factory.createElement("style");
    // fix case where certain rules are invalid - e.g: &:within(:not(.on)) does some
    // funny stuff.
    var ruleTexts = sheet.rules
        .map(function (rule) { return (0, paperclip_utils_1.stringifyCSSRule)(rule, { resolveUrl: resolveUrl }); })
        .map(function (text) {
        // OOF! This is expensive! This should be done in the rust engine instead. Not here!
        var isValid = ruleIsValid(text);
        // if (!isValid) {
        //   console.error(`invalid CSS rule: ${text}`);
        // }
        return isValid ? text : ".invalid-rule { }";
    });
    nativeElement.textContent = ruleTexts.join("\n");
    return nativeElement;
};
exports.createNativeStyleFromSheet = createNativeStyleFromSheet;
var createNativeTextNode = function (node, factory) {
    // fixes https://github.com/paperclipui/paperclip/issues/609
    return factory.createTextNode(entities.decode(node.value.replace(/[\s\r]+/g, " ")));
};
var createNativeElement = function (element, factory, resolveUrl, namespaceUri) {
    var nativeElement = element.tagName === "svg"
        ? document.createElementNS(XMLNS_NAMESPACE, "svg")
        : namespaceUri
            ? factory.createElementNS(namespaceUri, element.tagName)
            : factory.createElement(element.tagName);
    var childNamespaceUri = element.tagName === "svg" ? XMLNS_NAMESPACE : namespaceUri;
    for (var name_1 in element.attributes) {
        var value = element.attributes[name_1];
        if (name_1 === "src" && resolveUrl) {
            value = resolveUrl(value);
        }
        var aliasName = utils_1.ATTR_ALIASES[name_1] || name_1;
        nativeElement.setAttribute(aliasName, value);
    }
    for (var _i = 0, _a = element.children; _i < _a.length; _i++) {
        var child = _a[_i];
        nativeElement.appendChild((0, exports.createNativeNode)(child, factory, resolveUrl, childNamespaceUri));
    }
    // prevent redirects & vscode from asking to redirect.
    if (element.tagName === "a") {
        nativeElement.onclick = utils_1.preventDefault;
        nativeElement.onmouseup = utils_1.preventDefault;
        nativeElement.onmousedown = utils_1.preventDefault;
    }
    return nativeElement;
};
var createNativeFragment = function (fragment, factory, resolveUrl) {
    var nativeFragment = factory.createDocumentFragment();
    for (var _i = 0, _a = fragment.children; _i < _a.length; _i++) {
        var child = _a[_i];
        nativeFragment.appendChild((0, exports.createNativeNode)(child, factory, resolveUrl, nativeFragment.namespaceURI));
    }
    return nativeFragment;
};
