"use strict";
// TODO - move this to paperclip-utils
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateNode = exports.generateRandomPaperclipDocument = void 0;
var utils_1 = require("./utils");
var generateRandomPaperclipDocument = function (options) {
    var buffer = "";
    var context = { depth: 0 };
    buffer += (0, utils_1.randomArray)(options.minWidth, options.maxWidth)
        .map(function () {
        return (0, exports.generateNode)(options, context);
    })
        .join("\n");
};
exports.generateRandomPaperclipDocument = generateRandomPaperclipDocument;
var generateNode = function (options, context) {
    return (0, utils_1.pickRandom)([generateTextNode, generateFragment, generateBinding, generateElement], [1, 1, 1, 3])(options, context);
};
exports.generateNode = generateNode;
var generateTextNode = function (options, context) {
    return (0, utils_1.randomChars)(0, 20);
};
var generateFragment = function (options, context) {
    return "<fragment>".concat(generateChildNodes(options, context), "</fragment>");
};
var generateElement = function (options, context) {
    var tagName;
    while (1) {
        tagName = (0, utils_1.randomChars)(1, 3, 3);
        // Probably never going to happen, but no harm in checking
        if (!/img|br|import/.test(tagName)) {
            break;
        }
    }
    var buffer = "<" + tagName;
    buffer += (0, utils_1.randomArray)(options.minWidth, options.maxWidth)
        .map(function () {
        return " " + generateAttribute(options, context);
    })
        .join("");
    if ((0, utils_1.pickRandom)([true, false], [3, 1])) {
        buffer += ">";
        buffer += generateChildNodes(options, context);
        return buffer + "</".concat(tagName, ">");
    }
    else {
        // play with whitespace at end
        return buffer + (0, utils_1.pickRandom)([" />", "/>", "/>"]);
    }
};
var generateChildNodes = function (options, context) {
    if (context.depth >= options.maxDepth) {
        return "";
    }
    var buffer = "";
    if (context.depth < options.maxDepth) {
        buffer +=
            (0, exports.generateNode)(options, __assign(__assign({}, context), { depth: context.depth + 1 })) + "\n";
    }
    return buffer;
};
var generateAttribute = function (options, context) {
    return (0, utils_1.pickRandom)([generateKeyValueAttribute, generateBinding], [2, 1])(options, context);
};
var generateKeyValueAttribute = function (options, context) {
    var buffer = (0, utils_1.randomChars)(1, 3, 3);
    if ((0, utils_1.pickRandom)([true, false], [2, 1])) {
        return buffer + "=" + generateAttributeValue(options, context);
    }
    else {
        return buffer;
    }
};
var generateAttributeValue = function (options, context) {
    return (0, utils_1.pickRandom)([
        generateAttributeStringValue,
        generateAttributeDynamicStringValue,
        generateBinding
    ])(options, context);
};
var generateAttributeStringValue = function (options, context) {
    return "\"".concat((0, utils_1.randomChars)(0, 3, 3), "\"");
};
var generateAttributeDynamicStringValue = function (options, context) {
    var buffer = "\"";
    buffer += (0, utils_1.randomArray)(options.minWidth, options.maxWidth)
        .map(function () {
        return (0, utils_1.pickRandom)([
            (0, utils_1.randomChars)(1, 3),
            "{".concat((0, utils_1.randomChars)(1, 3), "}"),
            "{".concat((0, utils_1.randomChars)(1, 3), "?}") // optional
        ]);
    })
        .join(" ");
    return buffer + "\"";
};
var generateBinding = function (options, context) {
    return (0, utils_1.pickRandom)([
        "{".concat((0, utils_1.randomChars)(1, 3), "}"),
        "{".concat((0, utils_1.randomChars)(1, 3), "?}") // optional
    ]);
};
